from skimage import io, color
from PIL import Image

import numpy as np
import cv2


def correction( L0, shadow, highlight, shadow_tone, highlight_tone ):
    '''
    Given:
        `L0`: A w-by-h array of original luminance (range: [0, 1])
        `highlight`: A highlight number (default: 0) [-\infty, \infty]
        `shadow`: A shadow number (default: 0)

    Return:
        `L_correct`: A w-by-h array of corrected luminance
    '''
    ### We might (or might not) want this to be a function of highlight and shadow....
    ### We might need different values for different constraints
    
    # tone function --> formula: y = 1 / (1 + e^(25 * (t_s - L0) )
    tone_to_map = lambda t, lum: 1 / (1 + np.exp( 25 * ( t - lum ) ) )
    shadow_map = tone_to_map( shadow_tone, L0 )
    highlight_map = tone_to_map( highlight_tone, 1 - L0 )
    
    # apply correction on shadow and highlight (use some scaling to avoid fast trasition)
    correct = lambda lum, opt: - lum / ( np.exp( - opt / 10 ) * ( lum - 1 ) - lum )
    
    # feathering & reconstruction
    L_correct = ( 1 - shadow_map ) * L0 + shadow_map * correct( L0, shadow )
    L_correct = ( 1 - highlight_map ) * L_correct + highlight_map * correct( L_correct, highlight )
    return L_correct

def main():
    import argparse
    parser = argparse.ArgumentParser( description = 'shadow/highlight testing.' )
    parser.add_argument( 'img', help = 'The path to the input image.' )
    args = parser.parse_args()
    
    img = color.rgb2lab( np.array( Image.open( args.img ) ) / 255. )   
    
    L0 = img[:, :, 0] / 100
    L_ = correction( L0, -10, 25, 0.5, 0.1 )    # this means turn shadow up and fix highlight
    img[:, :, 0] = L_ * 100
    
    correct_img = ( color.lab2rgb( img ) * 255. ).clip( 0, 255 ).astype( np.uint8 )
    Image.fromarray( correct_img ).save( 'test-correct1.png' )
    
    
if __name__ == '__main__':
    main()